// ----------------------------------------------------------------------------
// This reference design and source code is being provided on an "as-is" basis 
// and as an accommodation, and therefore all warranties, representations or 
// guarantees of any kind (whether express, implied or statutory) including, 
// without limitation, warranties of merchantability, non-infringement, or  
// fitness for a particular purpose, are specifically disclaimed.
//
// This source code may only be used in an Altera programmable logic device
// and may not be distributed without permission from Macnica Americas, Inc.  
// It is provided free of royalties or fees of any kind.
// ----------------------------------------------------------------------------
//
// Description  | Simple frequncy detection for demonstration purposes
//              | Measures an input clock of the range 0.1MHz to 25.5MHz (approximately)
//              | with a resolution of 0.1MHz.  It is assumed the the signal
//              | being monitored is already synchronized to the logic clock.
//              | 
//              | The result is 8bit value (1-255) times the 0.1MHz resolution
//              | 
// Formatting   | Tabs set at 3 spaces.  Port names 'UPPER' case.
//              | Internal wires and 	isters are 'lower' case.
//              | Module names 'lower' case to match lower case filename.v
//              | Parameters are first character 'Upper' case.
//              | Active low signals are identified with '_n' or '_N'
//              | appended to the wire, 	ister, or port name.
//              | 
//              | _r suffix on a 	ister name indicates a simple
//              | re-	istering of another wire or 	ister.
//              | 
//              | _rr suffix on a 	ister name indicates a second
//              | re-	istering of a register.
//              | 
//              | _metastab indicates clock domain crossing
//
// ----------------------------------------------------------------------------

`timescale 1ns/1ps

//--------------------------
//	Declaration and ports
//-----------
module freqdetect (
	
	//global
	input					IN_CLOCK100MHZ,
	input					IN_RESET_N,
	
	//clk to monitor
	input					IN_CLK2MEASURE,
	
	//result
	output 	[7:0]		OUT_RESULT
);


//--------------------------
//	Regs and Wires
//-----------
	reg		[9:0]		cnt;
	reg		[9:0]		maxcnt;
	reg					sample;
	reg					sample_r;
	reg					sample_rr;
	wire					rising_edge;
	wire		[9:0]		quotient;
	wire		[9:0]		remainder;

//--------------------------
//	Locate Rising Edge Event
//-----------
always @(posedge IN_CLOCK100MHZ)
	begin
	if (!IN_RESET_N)
		begin
		sample <= 1'b0;
		sample_r <= 1'b0;
		sample_rr <= 1'b0;
		
		end
	else
		begin
		sample <= IN_CLK2MEASURE;
		sample_r <= sample;
		sample_rr <= sample_r;
		end
	end
assign rising_edge = (sample_r & !sample_rr);	


//--------------------------
//	The cycles counter
//-----------
always @(posedge IN_CLOCK100MHZ)
	begin
	if (!IN_RESET_N)
		begin
		cnt <= 10'b0;
		maxcnt <= 10'b0;
		end
	else
		begin
		if (rising_edge) //capture cycles. 
			begin
			maxcnt[9:0] <= cnt[9:0];
			cnt[9:0] <= 10'b1;
			end
		else if (cnt == 10'h3e8) //max count is 1000.  Hold at max if reached
			begin
			cnt <= 10'h3e8;
			end
		else	//increment
			begin
			cnt <= cnt + 1'b1;
			end
		end // if (rising edge)
	end // if (!N_RESET_N)

	
//--------------------------
//	Instance LPM_DIVIDE
// result in 0.1's of MHz = 1000 / maxcnt
//-----------
lpmdiv lpmdiv_inst (
	.clock ( IN_CLOCK100MHZ ),
	.denom ( maxcnt[9:0] ),
	.numer ( 10'h3e8 ),
	.quotient ( quotient[9:0] ),
	.remain ( remainder[9:0] )
	);


//--------------------------
//	Assign OUT_RESULT
//-----------
assign OUT_RESULT[7:0] = quotient[7:0];
	
	
endmodule